ACT_SONIC_DASH_CHARGE = allocate_mario_action(ACT_GROUP_STATIONARY | ACT_FLAG_IDLE | ACT_FLAG_ALLOW_FIRST_PERSON |
                                                  ACT_FLAG_PAUSE_EXIT | ACT_FLAG_SHORT_HITBOX | ACT_FLAG_INVULNERABLE |
                                                  ACT_FLAG_ATTACKING)
local peelRelease = audio_sample_load("PeelRelease.ogg")

--- @param m MarioState
local function spawn_terrain_particles(m)

    if (m.terrainSoundAddend == (SOUND_TERRAIN_WATER << 16)) then
        set_mario_particle_flags(m, PARTICLE_SHALLOW_WATER_SPLASH, 0);
    else
        if (m.terrainSoundAddend == (SOUND_TERRAIN_SAND << 16)) then
            set_mario_particle_flags(m, PARTICLE_DIRT, 0);
        elseif (m.terrainSoundAddend == (SOUND_TERRAIN_SNOW << 16)) then
            set_mario_particle_flags(m, PARTICLE_SNOW, 0);
        else
            set_mario_particle_flags(m, PARTICLE_DIRT, 0);
        end
    end
end

--- @param m MarioState
local function spawn__sonic_charge_particles(m)
    spawn_terrain_particles(m)
    set_mario_particle_flags(m, PARTICLE_DUST, 0);
end

--- @param m MarioState
local function act_sonic_dash_charge(m)
    local stats = _G.customMoves.stats_from_mario_state(m)
    if stats == nil then
        return
    end

    stationary_ground_step(m)
    if (m.controller.buttonDown & B_BUTTON) == 0 and m.actionArg > 9 then
        m.forwardVel = m.actionArg 
        set_mario_action(m, ACT_SONIC_DASH, 0)
        audio_sample_play(peelRelease, m.pos, 1)
        set_mario_particle_flags(m, PARTICLE_MIST_CIRCLE, 0)
        return
    end
    m.actionArg = m.actionArg + 11
    if m.actionArg > stats.sonic_dash_max_vel then
        m.actionArg = stats.sonic_dash_max_vel
    end

    smlua_anim_util_set_animation(m.marioObj, "JUMPBALL")
    gPlayerSyncTable[0].sonicAnimFrame = gPlayerSyncTable[0].sonicAnimFrame + 1 + m.actionArg / 165
    if gPlayerSyncTable[0].sonicAnimFrame >= m.marioObj.header.gfx.animInfo.curAnim.loopEnd then
        gPlayerSyncTable[0].sonicAnimFrame = 0
    end
    set_anim_to_frame(m, gPlayerSyncTable[0].sonicAnimFrame)
    spawn__sonic_charge_particles(m)

    m.faceAngle.y = m.intendedYaw - approach_s32(convert_s16(m.intendedYaw - m.faceAngle.y), 0, 0x800 / 10, 0x800 / 10)
    m.marioObj.header.gfx.angle.x = 0x8000 / 6
    return 0
end

hook_mario_action(ACT_SONIC_DASH_CHARGE, act_sonic_dash_charge)

smlua_anim_util_register_animation('JUMPBALL', 0, 0, 0, 0, 7,
    {0x0000, 0x00BD, 0x00F2, 0x0127, 0x00F2, 0x00BD, 0x0088, 0x0053, 0x0088, 0xFF96, 0xFFCB, 0x0000, 0x0035, 0x006A,
     0x0035, 0x0000, 0xFFCB, 0x0000, 0x3FFF, 0x4000, 0x4001, 0x8019, 0xC00F, 0xC010, 0xC011, 0x3FFF, 0x2000, 0xFFFF,
     0xDFFF, 0xC000, 0xDFFF, 0x0000, 0x2000, 0x0000, 0x3FFE, 0x3FFF, 0x3FFE, 0xFFF5, 0xC001, 0xC000, 0xC001, 0xFFFF,
     0xFFFF, 0xFFFF, 0xFFFF, 0x0000, 0xFFFF, 0x0000, 0x0000, 0xFFFF, 0x40A1, 0xFFFF, 0xFFFE, 0x24B2, 0xFFFF, 0xFFFE,
     0x2910, 0xFFFE, 0x0001, 0xC170, 0x0B72, 0xF361, 0x9657, 0x03DF, 0xF80A, 0xCC89, 0xFD4B, 0xF1AE, 0xD6F2, 0xFFFE,
     0x0001, 0xC170, 0xFA6C, 0x0D54, 0x965C, 0x009F, 0x04F0, 0xCF84, 0x05B7, 0x0663, 0xD414, 0x0000, 0xFFFF, 0xFFFF,
     0x0000, 0xFFFF, 0x0000, 0xFFFF, 0x0000, 0xFFFF, 0x0000, 0x0000, 0xFFFF, 0xFFFF, 0xFFFF, 0x0000, 0xFFFF, 0xBF5E,
     0x01A3, 0xF9BA, 0x778B, 0x0005, 0x009F, 0x48B2, 0x00C2, 0xFF36, 0xB37D, 0x0000, 0xFFFF, 0xFFFF, 0x0000, 0xFFFF,
     0x0000, 0xFFFF, 0x0000, 0xFFFF, 0x0000, 0x0000, 0xFFFF, 0xFFFF, 0xFFFF, 0x0000, 0xFFFF, 0xBF5E, 0xFD73, 0x05D4,
     0x770D, 0xFF49, 0xFE2F, 0x49A6, 0xFF67, 0x01FD, 0xB25A},
    {0x0001, 0x0000, 0x0008, 0x0001, 0x0008, 0x0009, 0x0008, 0x0011, 0x0008, 0x0019, 0x0008, 0x0021, 0x0007, 0x0029,
     0x0002, 0x0030, 0x0001, 0x0032, 0x0001, 0x0033, 0x0001, 0x0034, 0x0001, 0x0035, 0x0001, 0x0036, 0x0001, 0x0037,
     0x0001, 0x0038, 0x0001, 0x0039, 0x0001, 0x003A, 0x0001, 0x003B, 0x0001, 0x003C, 0x0001, 0x003D, 0x0001, 0x003E,
     0x0001, 0x003F, 0x0001, 0x0040, 0x0001, 0x0041, 0x0001, 0x0042, 0x0001, 0x0043, 0x0001, 0x0044, 0x0001, 0x0045,
     0x0001, 0x0046, 0x0001, 0x0047, 0x0001, 0x0048, 0x0001, 0x0049, 0x0001, 0x004A, 0x0001, 0x004B, 0x0001, 0x004C,
     0x0001, 0x004D, 0x0001, 0x004E, 0x0001, 0x004F, 0x0001, 0x0050, 0x0008, 0x0051, 0x0008, 0x0059, 0x0001, 0x0061,
     0x0001, 0x0062, 0x0001, 0x0063, 0x0001, 0x0064, 0x0001, 0x0065, 0x0001, 0x0066, 0x0001, 0x0067, 0x0001, 0x0068,
     0x0001, 0x0069, 0x0001, 0x006A, 0x0008, 0x006B, 0x0008, 0x0073, 0x0001, 0x007B, 0x0001, 0x007C, 0x0001, 0x007D,
     0x0001, 0x007E, 0x0001, 0x007F, 0x0001, 0x0080, 0x0001, 0x0081, 0x0001, 0x0082, 0x0001, 0x0083, 0x0001, 0x0084})
